unit ListBoxEx;

interface

uses
  Classes, SysUtils, LCLType,
  {$IFDEF DPI}Dpi.Graphics, Dpi.Controls, Dpi.StdCtrls, Dpi.Forms, Dpi.Common, System.UITypes{$ELSE}
  Graphics, Controls, StdCtrls, Forms{$ENDIF};

type

  { TListBoxEx }

  TListBoxEx = class(TListBox)
  protected
    procedure DrawItem(Control: TWinControl; Index: Integer; ARect: TRect;
      State: TOwnerDrawState); virtual;
  public
    constructor Create(TheOwner: TComponent); override;
  end;

procedure Register;


implementation

procedure Register;
begin
  RegisterComponents('C-evo', [TListBoxEx]);
end;

{ TListBoxEx }

procedure TListBoxEx.DrawItem(Control: TWinControl; Index: Integer; ARect: TRect;
  State: TOwnerDrawState);
var
  BackgroundColor: TColor;
  TextColor: TColor;
  NewRect: TRect;
const
  Orange = $5F77CB;
  Blue = $d77800;
begin
  if Index = 0 then begin
    Canvas.Brush.Color := Color;
    Canvas.Brush.Style := TBrushStyle.bsSolid;
    Canvas.FillRect(0, 0, Width, Height);
  end;

  if odSelected in State then begin
    BackgroundColor := Blue;
    TextColor := clWhite;
  end else begin
    BackgroundColor := clBlack;
    TextColor := Orange;
  end;
  Canvas.Brush.Color := BackgroundColor;
  Canvas.Brush.Style := TBrushStyle.bsSolid;
  Canvas.FillRect(ARect);
  Canvas.Font.Assign(Font);
  Canvas.Font.Color := TextColor;
  NewRect := Rect(ARect.Left + 1, ARect.Top + 1, ARect.Right - 1, ARect.Bottom - 1);
  Canvas.TextRect(NewRect, ARect.Left + 2, ARect.Top +
    (ARect.Height - Canvas.TextHeight(Items[Index])) div 2,
    Items[Index], Canvas.TextStyle);
  if odSelected in State then begin
    Canvas.Pen.Color := Orange;
    Canvas.Pen.Style := TPenStyle.psDot;
    Canvas.Brush.Style := TBrushStyle.bsClear;
    Canvas.Rectangle(NewRect);
  end;
end;

constructor TListBoxEx.Create(TheOwner: TComponent);
begin
  inherited;
  OnDrawItem := DrawItem;
  Style := TListBoxStyle.lbOwnerDrawVariable;
end;

end.

